import React, { createContext, useContext, useState, useEffect, } from 'react';
import { getNewHsl, getDegrees, isUpperCase, getHandleValue, getGradientType, computeSquareXY, compareGradients, computePickerPosition, } from './utils/utils.js';
import { low, high, getColors } from './utils/formatters.js';
import { config } from './constants.js';
import tinycolor from 'tinycolor2';
const { crossSize } = config;
const PickerContext = createContext(null);
export default function PickerContextWrapper({ value, bounds, children, onChange, squareSize, hideOpacity, squareHeight, }) {
    const offsetLeft = bounds?.x;
    const isGradient = value?.includes('gradient');
    const gradientType = getGradientType(value);
    const degrees = getDegrees(value);
    const degreeStr = gradientType === 'linear-gradient' ? `${degrees}deg` : 'circle';
    const colors = getColors(value);
    const indexedColors = colors?.map((c, i) => ({
        ...c,
        index: i,
    }));
    const currentColorObj = indexedColors?.filter((c) => isUpperCase(c.value))[0] ||
        indexedColors[0];
    const currentColor = currentColorObj?.value;
    const selectedColor = currentColorObj?.index;
    const currentLeft = currentColorObj?.left;
    const [tinyColor, setTinyColor] = useState(tinycolor(currentColor));
    const [inputType, setInputType] = useState('rgb');
    const { r, g, b, a: opacity } = tinyColor.toRgb();
    const { h: hue, s, l } = tinyColor.toHsl();
    const { s: hsvS, v: hsvV } = tinyColor.toHsv();
    const [internalHue, setInternalHue] = useState(Math.round(hue));
    const [x, y] = computeSquareXY(tinyColor.toHsv(), squareSize, squareHeight);
    const [previousColors, setPreviousColors] = useState([]);
    const [previousGraidents, setPreviousGradients] = useState([]);
    const [inFocus, setInFocus] = useState(null);
    const useragent = navigator.userAgent || '';
    const isMobile = useragent?.includes('Mobile');
    const internalOnChange = (newValue) => {
        if (newValue !== value) {
            if (isGradient) {
                if (!compareGradients(previousGraidents[0] || '', value)) {
                    setPreviousGradients([value, ...previousGraidents.slice(0, 8)]);
                }
            }
            else {
                setPreviousColors([value, ...previousColors.slice(0, 8)]);
            }
            onChange(newValue);
        }
    };
    useEffect(() => {
        setTinyColor(tinycolor(currentColor));
        setInternalHue(hue);
    }, [currentColor, hue]);
    const createGradientStr = (newColors) => {
        const sorted = newColors.sort((a, b) => a.left - b.left);
        const colorString = sorted?.map((cc) => `${cc?.value} ${cc.left}%`);
        internalOnChange(`${gradientType}(${degreeStr}, ${colorString.join(', ')})`);
    };
    const handleGradient = (newColor, left) => {
        const remaining = colors?.filter((c) => !isUpperCase(c.value));
        const newColors = [
            { value: newColor.toUpperCase(), left: left || currentLeft },
            ...remaining,
        ];
        createGradientStr(newColors);
    };
    const handleChange = (newColor) => {
        if (isGradient) {
            handleGradient(newColor);
        }
        else {
            internalOnChange(newColor);
        }
    };
    const handleOpacity = (e) => {
        const newO = getHandleValue(e) / 100;
        const newColor = `rgba(${r}, ${g}, ${b}, ${newO})`;
        handleChange(newColor);
    };
    const handleHue = (e) => {
        const newHue = getHandleValue(e) * 3.6;
        const newHsl = getNewHsl(newHue, s, l, opacity, setInternalHue);
        handleChange(newHsl);
    };
    // console.log("h", hue);
    // console.log("s", (x1 / squareSize) * 100);
    // console.log("v", 100 - (y1 / squareHeight) * 100);
    const handleColor = (e, ctx) => {
        const [x, y] = computePickerPosition(e);
        if (x && y) {
            const x1 = Math.min(x + crossSize / 2, squareSize - 1);
            const y1 = Math.min(y + crossSize / 2, squareHeight - 1);
            const newS = (x1 / squareSize) * 100;
            const newY = 100 - (y1 / squareHeight) * 100;
            const updated = tinycolor(`hsva(${hue}, ${newS}%, ${newY}%, ${opacity})`);
            handleChange(updated.toRgbString());
            // const [r, g, b] = ctx.getImageData(x1, y1, 1, 1).data
            // const rgbaData = `rgba(${r}, ${g}, ${b}, ${opacity})`
            // const newTc = tinycolor(rgbaData)
            // // @ts-expect-error some problem with certain tinycolor functions when importing module
            // const newHsl = newTc.toHsl(newTc)
            // const fixedH = tinycolor({
            //   h: hue,
            //   s: newHsl?.s,
            //   l: newHsl?.l,
            //   a: newHsl?.a,
            // })
            // handleChange(
            //   // @ts-expect-error some problem with certain tinycolor functions when importing module
            //   `rgba(${fixedH?._r}, ${fixedH?._g}, ${fixedH?._b}, ${fixedH?._a})`
            // )
        }
    };
    const setSelectedColor = (index) => {
        const newGradStr = colors?.map((cc, i) => ({
            ...cc,
            value: i === index ? high(cc) : low(cc),
        }));
        createGradientStr(newGradStr);
    };
    const addPoint = (e) => {
        const left = getHandleValue(e);
        const newColors = [
            ...colors.map((c) => ({ ...c, value: low(c) })),
            { value: currentColor, left: left },
        ]?.sort((a, b) => a.left - b.left);
        createGradientStr(newColors);
    };
    const deletePoint = () => {
        if (colors?.length > 2) {
            const formatted = colors?.map((fc, i) => ({
                ...fc,
                value: i === selectedColor - 1 ? high(fc) : low(fc),
            }));
            const remaining = formatted?.filter((_, i) => i !== selectedColor);
            createGradientStr(remaining);
        }
    };
    const nextPoint = () => {
        if (selectedColor !== colors?.length - 1) {
            setSelectedColor(selectedColor + 1);
        }
    };
    const handleClickFocus = (e) => {
        const formattedPath = e?.path?.map((el) => el.id);
        if (formattedPath?.includes('gradient-bar')) {
            setInFocus('gpoint');
        }
        else if (formattedPath?.includes('rbgcp-input')) {
            setInFocus('input');
        }
        else if (formattedPath?.includes('rbgcp-wrapper')) {
            setInFocus('picker');
        }
        else {
            setInFocus(null);
        }
    };
    useEffect(() => {
        window.addEventListener('click', handleClickFocus);
        // window.addEventListener('keydown', handleKeyboard)
        return () => {
            window.removeEventListener('click', handleClickFocus);
            // window.removeEventListener('keydown', handleKeyboard)
        };
    }, [inFocus, value]);
    const pickerContext = {
        x,
        y,
        s,
        l,
        r,
        g,
        b,
        hue,
        hsvS,
        hsvV,
        value,
        colors,
        degrees,
        inFocus,
        opacity,
        isMobile,
        onChange,
        addPoint,
        inputType,
        nextPoint,
        tinyColor,
        handleHue,
        setInFocus,
        isGradient,
        offsetLeft,
        squareSize,
        hideOpacity,
        handleColor,
        currentLeft,
        deletePoint,
        internalHue,
        squareHeight,
        setInputType,
        gradientType,
        handleChange,
        currentColor,
        selectedColor,
        handleOpacity,
        setInternalHue,
        previousColors,
        handleGradient,
        setSelectedColor,
        internalOnChange,
        previousGraidents,
    };
    return (React.createElement(PickerContext.Provider, { value: pickerContext }, children));
}
export function usePicker() {
    const pickerContext = useContext(PickerContext);
    if (!pickerContext) {
        throw new Error('usePicker has to be used within <PickerContext.Provider>');
    }
    return pickerContext;
}
